/************************************************************************/
/*				        	                        */
/*	  PmodDA4 Demo Project 		                                */
/*					    	                        */
/************************************************************************/
/*	Author: James Colvin						*/
/*	Copyright 2014, Digilent Inc.					*/
/************************************************************************/
/*  File Description: 			             		        */
/*					                        	*/
/* This file implements a simple demo application that demonstrates     */
/* how to setup and use the PmodDA4.				        */
/*									*/
/*	Functionality:							*/
/*									*/
/* In the setup() function, the PmodDA4 is initialized through a        */
/* created function so that it uses its own internal reference          */
/* voltage.                                                             */
/*                                                                      */
/* In the loop() function, the application repeatedly increases and     */
/* decreases the physical value to be reconstructed by the DA converter.*/
/*					       	                        */
/*	Required Hardware:		                                */
/*	  1. PIC32 based Microcontroller    	                        */
/*	  2. PmodDA4 - plugged into an SPI compatible pins      	*/
/*			                                                */
/************************************************************************/
/*  Revision History:			        			*/
/*					                        	*/
/*	08/28/2014(JamesC): created	       			        */
/*                                                                      */
/*                                                                      */
/*					      	                        */
/************************************************************************/

/* -------------------------------------------------------------------- */
/*		        Include File Definitions                     	*/
/* -------------------------------------------------------------------- */
#include <SPI.h>

/* -------------------------------------------------------------------- */
/*		            Global Variables                     	*/
/* -------------------------------------------------------------------- */
// none

/* -------------------------------------------------------------------- */
/*	               Procedure Definitions	                        */
/* -------------------------------------------------------------------- */
/***	setup
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		 Performs basic initialization.		
/*** ---------------------------------------------------------- ***/
void setup()
{
  // set SPI pins to be outputs
  pinMode(SS, OUTPUT);
  pinMode(MOSI, OUTPUT);
  pinMode(SCK, OUTPUT);
  // initialize SPI
  SPI.begin(); // corresponds to SPI2 - available on connector J8
  SPI.setBitOrder(MSBFIRST); // during the transfer of data, send the most significant bit first
  SPI.setDataMode(SPI_MODE0); // Clock starts at a LOW value and transmits data on the falling edge of the clock signal
  
  digitalWrite(SS, LOW); // selecting the DAC
  setUpDAC(); // defined function call to set up the DA4 to use its internal voltage reference
  digitalWrite(SS, HIGH); // deselecting the DAC
}

/*** ---------------------------------------------------------- ***/
/***	loop
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Main program module. Enters the main program loop.
/*** ---------------------------------------------------------- ***/
void loop()
{
  // tells the DAC in the required 32 bits to turn on all outputs to the full analog value  
  digitalWrite(SS, LOW); // selects the DAC
  sendAddressAndValue(15, 4095); // full analog signal
  digitalWrite(SS, HIGH); // deselects the DAC
  delay(500); // delays for 500 milliseconds
  
  digitalWrite(SS, LOW);
  sendAddressAndValue(15, 3500); // slightly less than full analog signal
  digitalWrite(SS, HIGH);
  delay(500);
  
  digitalWrite(SS, LOW);
  sendAddressAndValue(15, 3250); // slightly less again
  digitalWrite(SS, HIGH);
  delay(500);
  
  digitalWrite(SS, LOW);
  sendAddressAndValue(15, 3000); // and again
  digitalWrite(SS, HIGH);
  delay(500);

}//end of loop

/* -------------------------------------------------------------------- */
/*	               User Defined Functions	                        */
/* -------------------------------------------------------------------- */

/***	sendAddressAndValue
**
**	Parameters:
**		Two integers
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		 Checks to see if the given address and value
**                    are valid; if they are the information is
**                    sent to the DAC. If not, LD5 on the uC32
**                    microcontroller is blinked rapidly		
/*** ---------------------------------------------------------- ***/

void sendAddressAndValue(int address, int value){
  //valid address values are between 0 and 7; 0 for A, 1 for B, and so on.  
  //An address value of 15 will provide power to all of the outputs simultaneously
  
  if(((address>=0 && address<=7) || address==15) && (value>0 && value<=4095)){
    SPI.transfer(0b00000011); // 4 bits of junk and a command to update DAC register
    delay(1); // small time delay to ensure the DAC processes the data
    int value1 = highByte(value); // collecting the upper half of the value
    int value2 = lowByte(value); // collecting the lower half of the value
    int addressAndValue1 = (address<<4) + value1; // combining the address and value into one piece
    SPI.transfer(addressAndValue1); // the 4 bits of address and the upper half of the value
    delay(1); // small time delay to ensure the DAC processes the data
    SPI.transfer(value2); // the lower half of the value
    delay(1); // small time delay to ensure the DAC processes the data
    SPI.transfer(0);//junk data
    delay(1); // small time delay to ensure the DAC processes the data
    } // end of if statement
    
  else{
    SPI.transfer(0b00000011); // 4 bits of junk and a command to update DAC register
    delay(1); // small time delay to ensure the DAC processes the data
    SPI.transfer(0b11110000); // all addresses and a value of zero
    delay(1); // small time delay to ensure the DAC processes the data
    SPI.transfer(0); // a value of zero
    delay(1); // small time delay to ensure the DAC processes the data
    SPI.transfer(0);//junk data
    delay(1); // small time delay to ensure the DAC processes the data
    for(int i=0; i<8; i++){
      digitalWrite(43, HIGH);
      delay(25);
      digitalWrite(43, LOW);
      delay(25); 
      }
    } // end of else statement
    
} //end of sendAddressAndValue


/***	setUpDAC
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Sets up the DAC to use the internal reference
**                  voltage, also using some previous defined 
**                  functions.		
/*** ---------------------------------------------------------- ***/

void setUpDAC(){
  SPI.transfer(0b00001000);//4 bits of junk and a command to change reference voltage
  delay(3); // small time delay to ensure DAC processes the data
  SPI.transfer(0);//sending more values since it doesn't care about these (thanks to the Command) and we need 32 bits total
  delay(3);
  SPI.transfer(0);//same reason
  delay(3);
  SPI.transfer(0b00000001);//7 bits of ignored data and a final "1" to indicate to use the internal reference
  delay(3);
  digitalWrite(SS, HIGH);
  delay(1);
  digitalWrite(SS, LOW);
  //turning all the outputs off to begin with
  SPI.transfer(0b00000011); // 4 bits of junk and a command to update DAC register
  delay(1); // small time delay to ensure the DAC processes the data
  SPI.transfer(0b11110000); // all addresses and a value of zero
  delay(1); // small time delay to ensure the DAC processes the data
  SPI.transfer(0); // a value of zero
  delay(1); // small time delay to ensure the DAC processes the data
  SPI.transfer(0);//junk data
  delay(1); // small time delay to ensure the DAC processes the data
}

